package com.jauharteam.ojek.webapp;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.jauharteam.ojek.ojek.UserService;
import com.ojek.common.User;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;

import static com.ojek.common.util.RestUtil.httpPost;

/**
 * Created by dery on 11/7/17.
 */
public class RegisterServlet extends WebappServlet {
    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        if (checkIsLoggedIn(req)) {
            resp.sendRedirect(config.getBaseUrl() + "/profile");
            return;
        }
        req.getRequestDispatcher("/signup.jsp").forward(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        String name = req.getParameter("name");
        String phone = req.getParameter("phone");
        String driverStr = req.getParameter("driver");

        //===================================================//
        String remoteAddr = "";
        if (req!= null) {
            remoteAddr = req.getHeader("X-FORWARDED-FOR");
            if (remoteAddr == null || "".equals(remoteAddr)) {
                remoteAddr = req.getRemoteAddr();
            }
        }

        String userAgent = req.getHeader("User-Agent");
        String ipAddress = remoteAddr;

        System.out.println("webapp-user " + userAgent);
        System.out.println("webapp-ip " + ipAddress);

        Map<String, String> newReq = getAllParam(req);
        newReq.put("User-Agent", userAgent);
        newReq.put("Ip-Address", ipAddress);
        //===================================================//

        Boolean isDriver = false;
        if (driverStr != null && driverStr.length() > 0)
            isDriver = driverStr.equals("1");

        if (checkIsLoggedIn(req)) {
            req.setAttribute("errorMessage", "User already logged in");
            resp.sendRedirect(config.getBaseUrl() + "profile");
            return;
        }

        if (config != null) {
            // Do post request
            String responseStr = httpPost(config.getIdentityRestPath() + "/register", newReq);
            if (responseStr == null)
                req.setAttribute("errorMessage", "Internal server error.");
            else {
                ObjectMapper objectMapper = new ObjectMapper();
                JsonNode responseNode = objectMapper.readTree(responseStr);
                if (responseNode != null && responseNode.get("success").asBoolean()) {
                    String accessToken = responseNode.get("accessToken").asText();
                    String refreshToken = responseNode.get("refreshToken").asText();
                    String expiredToken = responseNode.get("expired").asText();

                    resp.addCookie(new Cookie("accessToken", accessToken));
                    resp.addCookie(new Cookie("refreshToken", refreshToken));
                    resp.addCookie(new Cookie("tokenExpired", expiredToken));

                    UserService userService = getOjekUserService();
                    User user = getIdentityService().getUserByToken(accessToken);
                    user.setName(name);
                    user.setPhone(phone);
                    user.setDriver(isDriver);
                    if(userService.addUser(user)) {
                        resp.sendRedirect(config.getBaseUrl() + "profile");
                        return;
                    } else {
                        req.setAttribute("errorMessage", "Internal server error.");
                    }
                } else {
                    String errorMessage = responseNode.get("errorMessage").asText();
                    if (errorMessage == null)
                        errorMessage = "Please fill the correct data";
                    req.setAttribute("errorMessage", errorMessage);
                }
            }
        } else {
            req.setAttribute("errorMessage", "Internal server error.");
        }
    }
}
